from .utils import *
from .style import Style, BOLD, FG_RED, FG_YELLOW, RESET, bg_hex, bg_rgb, fg_hex, fg_rgb
from .input import Input, inp
from .output import Output, prt, NbCmdIO

__version__ = "1.8.72"

__all__ = ['Output', 'Input', 'NbCmdIO', 'prt', 'inp', 'TIMER', 'Style', 'BOLD', 'FG_RED', 'FG_YELLOW', 'RESET', 'RGB', 'bg_hex', 'bg_rgb', 'fg_hex', 'fg_rgb']